import { compile, preprocess as preprocessor } from 'svelte/compiler';
import createLoader from 'create-esm-loader';
import { cwd } from 'node:process';
import { extname, parse } from 'node:path';
import { fileURLToPath } from 'node:url';
import semver from 'semver';
const NAME = 'esm-loader-svelte';
const EXT = '.svelte';
let svelteKitPreprocess;
try {
    const svelteKitConfig = (await import(`${cwd()}/svelte.config.js`)).default;
    svelteKitPreprocess = svelteKitConfig.preprocess;
}
catch (error) { } // eslint-disable-line no-empty
// create-esm-loader config
const config = {
    resolve: (specifier, options) => {
        const { debug, parentURL } = options;
        const { href, pathname } = new URL(specifier, parentURL);
        if (extname(pathname) === EXT) {
            if (debug)
                console.log(`[${NAME}] resolve: ${specifier}`);
            return {
                format: semver.gte(process.versions.node, '16.12.0')
                    ? 'module' // node>=16.12
                    : undefined,
                url: href,
            };
        }
    },
    format: (url, options) => {
        const { debug } = options;
        const { pathname } = new URL(url);
        if (semver.gte(process.versions.node, '16.12.0'))
            return undefined;
        // node<16.12
        if (extname(pathname) === EXT) {
            if (debug)
                console.log(`[${NAME}] format: ${url}`);
            return { format: 'module' };
        }
    },
    transform: async (source, options) => {
        const { debug, preprocess, url } = options;
        const { pathname } = new URL(url);
        if (extname(pathname) != EXT)
            return undefined;
        if (debug)
            console.log(`[${NAME}] transform: ${url}`);
        const name = parse(url).name.replace(/[^A-Za-z0-9]/g, '');
        const filename = fileURLToPath(url);
        let sourceStr = String(source);
        // preprocess
        let processed = { map: undefined };
        const preprocessConfig = preprocess || svelteKitPreprocess;
        if (preprocessConfig) {
            processed = await preprocessor(sourceStr, preprocessConfig, {
                filename,
            });
            sourceStr = String(processed.code);
        }
        // compile
        const { js, warnings } = compile(sourceStr, {
            name: name[0].toUpperCase() + name.substring(1),
            filename,
            sourcemap: preprocessConfig ? processed.map : undefined,
        });
        warnings.forEach((warning) => {
            console.warn(`[${NAME}] svelte compile() warnings for ${warning.filename}:`);
            console.warn(warning.message);
            console.warn(warning.frame);
        });
        // result
        const { code, map } = js;
        const result = [code, `//# sourceMappingURL=${map.toUrl()}`].join('\n');
        return { source: result };
    },
};
export default config;
// node loader
export const { resolve, load, getFormat, // node<16.12
getSource, // node<16.12
transformSource, // node<16.12
 } = await createLoader(config);
